import unittest
import sys
sys.path.insert(0, sys.path[0][:-4])
from MDS_Big_Adventure.scene.level import level

class TestLevel(unittest.TestCase):
    
    @classmethod
    def setUpClass(cls):
        print("")
    
    def setUp(self):
        self.l = level(0,["plot A","plot B"],"monster")
        
    def test_getStatus(self):
        self.assertEqual(self.l.getStatus(),0)
        self.assertNotEqual(self.l.getStatus(),1)
        self.assertNotEqual(self.l.getStatus(),50)
        self.assertNotEqual(self.l.getStatus(),20)
        
    def test_getLoc(self):
        self.assertEqual(self.l.getLoc(),0)
        self.assertNotEqual(self.l.getLoc(),200)
        self.assertNotEqual(self.l.getLoc(),-1)
        self.assertNotEqual(self.l.getLoc(),100)
        
    def test_getplot(self):
        self.assertEqual(self.l.getplot(),"plot A")
        self.l.visit()
        self.assertEqual(self.l.getplot(),"plot B")
    def tearDown(self):
        print("")
        
    @classmethod
    def tearDownClass(cls):
         print("")
         
    unittest.main(argv=[''], verbosity=2, exit=False)   




class level():
    __status = 0
    __locked = 1

    #initialize
    def __init__(self, location, lines, alien):
        self.__location = location
        self.__lines = lines
        self.__alien = alien

    #return location    
    def getLoc(self):
        return self.__location

    #set status to 1 if the level is visited
    def visit(self):
        self.__status = 1

    #return the status of current level
    def getStatus(self):
        return self.__status
    
    #check if current level is visited
    def isVisited(self):
        return self.__status == 0

    #return alien at the current level
    def getAlien(self):
        return self.__alien
    
    #return the lines based on level status
    def getplot(self):
        if self.isVisited():
            return self.__lines[0]
        else:
            return self.__lines[1]

    #unlock the level if alien is dead
    def unlock(self):
        if self.__alien.isdead():
            self.__locked = 0

    #check if the level is locked
    def islocked(self):
        return self.__locked == 1



class character():

    #initialize
    def __init__(self, health, power):
        self.health = health
        self.power = power

    #display health and power
    def display(self):
        return "Health: {} power: {}".format(self.health, self.power)

    #return the power value
    def fight(self):
        return self.power 

    #deduct health by the alien attack
    def wound(self, damage):
        self.health -= damage
        if self.health <= 0:
            self.health = 0



import unittest
import sys
sys.path.insert(0, sys.path[0][:-4])
from MDS_Big_Adventure.character.character import character

class TestCharacter(unittest.TestCase):
    
    @classmethod
    def setUpClass(cls):
        print("")
    
    def setUp(self):
        self.c = character(100,50)
        
    def test_fight(self):
        self.assertEqual(self.c.fight(),50)
        self.assertNotEqual(self.c.fight(),100)
        self.assertNotEqual(self.c.fight(),0)
        self.assertNotEqual(self.c.fight(),-100)
        
    def test_display(self):
        self.assertEqual(self.c.display(),"Health: 100 power: 50")
        self.assertNotEqual(self.c.display(),"Health: 100 power: 100")
        self.assertNotEqual(self.c.display(),"power: 50")
        self.assertNotEqual(self.c.display(),"100; 50")
    def test_wound(self):
        self.c.wound(10)
        self.assertEqual(self.c.health, 90)
        self.c.wound(100)
        self.assertEqual(self.c.health, 0)
    def tearDown(self):
        print("")
        
    @classmethod
    def tearDownClass(cls):
         print("")
         
    unittest.main(argv=[''], verbosity=2, exit=False)   



import unittest
import sys
sys.path.insert(0, sys.path[0][:-4])
from MDS_Big_Adventure.item.sparkLance import sparkLance

class TestSparkLance(unittest.TestCase):
    
    @classmethod
    def setUpClass(cls):
        print("")
    
    def setUp(self):
        self.s = sparkLance("sparkLance",20,30,0)
        
    def test_getstatus(self):
        self.assertEqual(self.s.getstatus(),0)
        self.assertNotEqual(self.s.getstatus(),1)
        self.assertNotEqual(self.s.getstatus(),30)
        self.assertNotEqual(self.s.getstatus(),2)
        
    def test_getplot(self):
        self.assertEqual(self.s.getplot(),["""
                                          As soon as you touched the Spark Lance, it started glowing and immersed you in light.
                                          You were startled at first, but soon you felt something, something strange but familiar.
                                          It felt like those sunny afternoons, when you look out from the window, 
                                          everything is so quite under the clear blue sky, you feel peaceful. 
                                          It then felt like the times when you finished the very last exam of the term, you are relieved, and hope
                                          is coming back to you agian.
                                          After that, you felt courage, it was like recieving an quiz score better than you had hoped, you started to believe
                                          in yourself.

                                          Suddenly, You heard a voice in the light: 

                                          "The only way to save the world, is you become the light"

                                          Yes, I can be the light! You thought.

                                          You raised the Spark Lance high:

                                          Ultraman !!!!!!!!!!

                                          ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~Exciting BGM playing ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                                          *********************************************Fancy visual effects *************************************************

                                          You transformed into Ultraman with the power of the light, you now have special powers to save the world.
                          """,
                """
                                           "It is you..." The Tim Hortons staff said slowly.
                                           "Many years ago an old lady came here and ordered turkey bacon club,
                                           I told her we only have breakfast at this branch, but she came back again
                                           the following day and ordered the same thing. After I told her the
                                           that we really don't sell turkey bacon here, she smiled and gave something to me, wait here please." 
                                           The staff hurried into the storage room. He came back with a strange looking artifact in his hand.
                                           
                                           Strangly, your attantion is captured by it, as if it is speaking to you.
                                           
                                           "This is Spark Lance," said the staff, "the old lady said if one day this world is
                                           in trouble, someone would come here to order turkey bacon club, I should tell that
                                           person the same thing I told her at the first time. If that person came back again, 
                                           I should give this to them." 
                                           
                                           The Tim Horton's staff handed the artifact to you"""])
        self.assertNotEqual(self.s.getplot(),200)
        self.assertNotEqual(self.s.getplot(),-1)
        self.assertNotEqual(self.s.getplot(),100)
        
    def test_check(self):
        self.s.check()
        self.assertEqual(self.s.getstatus(), 1)
    
    def test_betaTransform(self):
        class p():
            def __init__(self, health):
                self.health = health
        self.p = p(10)
        
        self.s.betaTransform(self.p)
        
        self.assertEqual(self.p.health, 10*20)
        self.s.betaTransform(10)
        
    def tearDown(self):
        print("")
        
    @classmethod
    def tearDownClass(cls):
         print("")
         
    unittest.main(argv=[''], verbosity=2, exit=False)   




import unittest
import sys
sys.path.insert(0, sys.path[0][:-4])
from MDS_Big_Adventure.character.alien import alien

class TestAlien(unittest.TestCase):
    
    @classmethod
    def setUpClass(cls):
        print("")
    
    def setUp(self):
        self.a = alien(100,50,"monster",20)
        
    def test_getName(self):
        self.assertEqual(self.a.getName(),"monster")
        self.assertNotEqual(self.a.getName(),100)
        self.assertNotEqual(self.a.getName(),50)
        self.assertNotEqual(self.a.getName(),20)
        
    def test_dropMoney(self):
        self.assertEqual(self.a.dropMoney(),20)
        self.assertNotEqual(self.a.dropMoney(),200)
        self.assertNotEqual(self.a.dropMoney(),-1)
        self.assertNotEqual(self.a.dropMoney(),100)
    def test_die(self):
        self.a.die()
        self.assertEqual(self.a.health, 0)
    def test_isdead(self):
        self.a.die()
        self.assertTrue(self.a.isdead())
    def test_wound(self):
        self.a.wound(10)
        self.assertEqual(self.a.health, 90)
        self.a.wound(100)
        self.assertEqual(self.a.health, 0)
        
        
    def tearDown(self):
        print("")
        
    @classmethod
    def tearDownClass(cls):
         print("")
         
    unittest.main(argv=[''], verbosity=2, exit=False)   




import unittest
import sys
sys.path.insert(0, sys.path[0][:-4])
from MDS_Big_Adventure.character.player import player
from MDS_Big_Adventure.exceptions import InvalidPriceException
class TestPlayer(unittest.TestCase):
    
    @classmethod
    def setUpClass(cls):
        print("")
    
    def setUp(self):
        self.p = player(100,50,1,1)
        
    def test_showMoney(self):
        self.assertEqual(self.p.showMoney(),1)
        self.assertNotEqual(self.p.showMoney(),100)
        self.assertNotEqual(self.p.showMoney(),50)
        self.assertNotEqual(self.p.showMoney(),-100)
        
    def test_getidentity(self):
        self.assertEqual(self.p.getidentity(),1)
        self.assertNotEqual(self.p.getidentity(),0)
        self.assertNotEqual(self.p.getidentity(),2)
        self.assertNotEqual(self.p.getidentity(),3)
        
    def test_pay(self):
        self.assertEqual(self.p.pay(-0.1), "Price can't be negative")
        self.assertEqual(self.p.pay("1"), "Price has to be a number")
        self.p.pay(1)
        self.assertEqual(self.p.money, 0)
        
    
    
    def test_getMoney(self):
        self.p.getMoney(5)
        self.assertEqual(self.p.showMoney(), 6)
        self.p.getMoney("abc")
        self.assertEqual(self.p.showMoney(), 6)
        
    def test_advance(self):
        self.p.advance()
        self.assertEqual(self.p.position, 1)
        self.p.advance()
        self.p.advance()
        self.p.advance()
        self.p.advance()
        self.assertEqual(self.p.position, 4)
        
    def test_retreat(self):
        self.p.retreat()
        self.assertEqual(self.p.position, 0)
        self.p.advance()
        self.p.retreat()
        self.assertEqual(self.p.position, 0)
        
    def test_wound(self):
        self.p.wound(10)
        self.assertEqual(self.p.health, 90)
        self.p.wound(100)
        self.assertEqual(self.p.health, 0)
        
    def tearDown(self):
        print("")
        
    @classmethod
    def tearDownClass(cls):
         print("")
         
    unittest.main(argv=[''], verbosity=2, exit=False)   




from .level import level

#inheritance from class level
class timHortons(level):

    #initialize
    def __init__(self, location, lines, alien, inventory):
        level.__init__(self, location, lines, alien)
        self.visit()
        self.unlock()
        self.__inventory = inventory

    #update item amount when the item is sold 
    def sell(self, item):
        try:
            item.setamount(item.getamount()-1)
        except Exception as ex:
            
            return "something is wrong with our stock"

    #check item stock
    def checkStock(self, name):
        try:
            if name.isdigit():
                raise ValueError
            for r in self.getInv():
                if r.getname() == name and r.getamount() > 0:
                    return r
                elif r.getname() == name and r.getamount() == 0:
                    return 1
                elif name == "turkey bacon club":
                    return 3
            return 2
        except ValueError:
            return 2

    #return inventory
    def getInv(self):
        return self.__inventory




from .character import character

#inheritance from class character
class alien(character):

    #initialize
    def __init__(self, health, power, name, money):
        character.__init__(self, health, power)
        self.__name = name
        self.__money = money

    #return alien name    
    def getName(self):
        return self.__name

    #set alien name
    def setName(self, string):
        self.__name = string

    #change alien attributes when alien is dead
    def die(self):
        self.health = 0
        self.power = 0
        self.setName("alien dead body")
        self.__money = 0
        

    #return if the alien is dead
    def isdead(self):
        return self.health == 0

    #display
    def display(self):
        if self.isdead():
            return "(x__x) " + character.display(self)
        else:
            return "(╰_╯) " + character.display(self)

    #when alien is dead, drop the money
    def dropMoney(self):
        return self.__money



from setuptools import setup, find_packages
  
setup(
    name="MDS_big_advantrue", 
    version="0.1",
    packages=find_packages(exclude=['tests*']),
    license='MIT',
    description="A small example package",
    url='https://github.com/chen-yuhong/DATA533-step3.git',
    author="Henry(Yiheng) Luo; Yuhong Chen",    
    author_email="henryluo@student.ubc.ca"
)



import unittest
import sys
sys.path.insert(0, sys.path[0][:-4])
from MDS_Big_Adventure.scene.timhortons import timHortons
from MDS_Big_Adventure.character.alien import alien
from MDS_Big_Adventure.item.food import item

class TestTimhortons(unittest.TestCase):
    
    @classmethod
    def setUpClass(cls):
        print("")
    
    
    def setUp(self):
        self.i = item("bacon",20,30,2,2)
        self.i2 = item("potato", 20, 20, 10, 0 )
        self.t = timHortons(0,["plot A","plot B"],alien(0,0,"a",0),[self.i,self.i2])
        
    def test_getInv(self):
        self.assertEqual(self.t.getInv(),[self.i, self.i2])
        self.assertNotEqual(self.t.getInv(),"bread")
        self.assertNotEqual(self.t.getInv(),"milk")
        self.assertNotEqual(self.t.getInv(),20)
    
    def test_checkStock(self):
        self.assertEqual(self.t.checkStock("bacon"), self.i)
        self.assertEqual(self.t.checkStock("skdhg"), 2)
        self.assertEqual(self.t.checkStock("turkey bacon club"), 3)
        self.assertEqual(self.t.checkStock("potato"), 1)
        
    def test_sell(self):
        self.assertEqual(self.t.sell("bacon"), "something is wrong with our stock")
        self.t.sell(self.i)
        self.assertEqual(1, self.i.getamount())
        
    
    def tearDown(self):
        print("")
        
    @classmethod
    def tearDownClass(cls):
         print("")
         
    unittest.main(argv=[''], verbosity=2, exit=False)   




import random

from .item.food import item
from .item.sparkLance import sparkLance
from .scene.level import level
from .scene.timhortons import timHortons
from .character.alien import alien
from .character.player import player


class play():

    #beginning scripture
    script0 = """
        It is an ordinary day in the MDS program, you finished the morning classes
        and went to Tim Hortons for lunch. All of a sudden, you hear a big bang sound
        tearing through the sky. You looked up, UFOs! Alien vessels are hovering over the
        science building, aliens are landing on the roof! People around you started screaming
        and running. You suddenly realized something important, your laptop is still in the building,
        and you haven't submitted the lab in it! You need to go get your laptop!
        
        You are at Tim Hortons
        What is your next move?
        
        """

    #initialize
    def __init__(self):
        #player with 100 health, 100 power, 0 moeny, general identity
        self.p1 = player(100, 100,0,0)

        #item named hashbrown with health bonus 10, power bonus 5, price $3, amount 2
        self.i1 = item("hashbrown", 10, 5, 3, 2)

        #item named maple bacon bagel belt with health bonus 30, power bonus 15, price $7, amount 1
        self.i2 = item("maple bacon bagel belt", 30, 15, 7, 1)

        #item named sausage farmer's wrap with health bonus 40, power bonus 25, price $8, amount 1
        self.i3 = item("sausage farmer's wrap", 40, 25, 8, 1)

        #item named Spark Lance with health multiply 100, power multiply 100, status 0
        self.sparklance = sparkLance("Spark Lance", 100, 100, 0)

        #create tim horton at location 0 with 3 items and a non-aggressive alien
        self.tim = timHortons(0, ["you are in tim's","you are in tim's"], alien(0,0,"place holder",0), [self.i1,self.i2,self.i3])

        #alien named alien scientist with health 100, power 80, money 3
        self.a1 = alien(100, 80, "alien scientist",3)

        #alien named alien infantry soldier with health 120, power 90, money 8
        self.a2 = alien(120, 90, "alien infantry soldier",8)

        #alien named alien infantry soldier with health 150, power 100, money 10
        self.a3 = alien(150, 100, "alien elite commando",10)

        #alien named alien captain with health 200, power 150, money 15
        self.a4 = alien(200,150, "alien captain", 15)

        #scriptures for each level
        self.cr1 = level(1, ["""
                                You are on the first floor of the science building
                                It is a huge mess. Most of the students have escaped from the building, some are still running around
                                looking for the exit. There is one still studying... wait... no... that's not a student!
                                A tall scientist looking alien is scanning the papers on the wall. "#$#^%*&statist*csss, %$#!@$%...", he is murmuring
                                to himself. Apparently, he is stealing, or mocking, the scientific progress of Earth. Either way, you need to
                                stop him! """,
                             """
                               You returned to the first floor, it is very quiet here
                              "turkey bacon club ~~~" You heard an indistinctive whisper, it might just be an illusion"""], self.a1)
        self.cr2 = level(2, ["""
                                You are on the 2nd floor
                                You found a soldier looking alien standing in front of he garbage bins, holding a weapon.
                                He looks puzzled, as if he is confused what garbage should go to which bin. "&*@&#(@)#&?", he murmured
                                with frustration. You can seize this chance to attack him by surprise!""",
                             """
                                 you returned to the second floor, there is nobody here, only over turned garbage bins
                                "turkey bacn club~~", you think you heard a whisper, but there is nobody here...
                                """], self.a2)
        self.cr3 = level(3, ["""
                                You are on the third floor
                                There is an alien in the chemistry lab drinking random chemicals.
                                "&#*!!!!!", he probably drank one of the acids. As he madly wiped his tongue, he spotted you!
                                """,
                             """
                                 you returned to the chemistry labs
                                 """], self.a3)
        self.cr4 = level(4, ["""
                                You finally entered SCI 396
                                An alien with fancy armor is in the classroom. He is...well...swiveling on one of the swivel chairs.
                                "Ahhh!!!! Well! come!! warrior of this PLANET!!!! I have been MONITORING your PROGRASS!!!" 
                                He got up and shouted to you in human language.
                                
                                "You can speak our language? What were you doing on the chair? And why are you shouting?" You are puzzled.
                                
                                "My universal TRANSLATOR have picked up some preliminary samples from the HUMANS screaming and running AROUND!!!
                                But the samples are limited to mostly shouting and CRYING! in despair， as you can imagine!!!
                                At least it WORKS for now!!! I AM! the captain OF the &^*(*^ vessel *((*^, We are ON our pilgrimage!!!"
                                
                                "OK, I see... what pilgrimage? And I am not a warrior, I just need my laptop to finish my assignment,
                                it is due in a few ours. "
                                
                                "Oh, what a brutal civilization! You torture your children at this kindergarten and force your warriors to do
                                children's work for no reason!" He is not shouting as much now, his universal translator must have sampled your
                                normal speaking.
                                
                                "What are you talking about, this is not a kindergarten, it is a university, and we don't torture our children!"
                                
                                "What?!, you are teaching Maximum likelihood estimation at a university? My 2-year-old daughter can do that.
                                And these swivel chairs are not for swiveling students who answered the questoin wrong for punishment? And what about
                                those cafeteria next door, the terrible foods are not for torturing the students?" Said the alien.
                                
                                "What? No, no the equations... the chairs...ah I don't even want to explain. And those labs next door are not cafeteria,
                                there are dangerous chemicals in it! Well... but you might be right about the foods on campus... "
                                
                                "Who would put chemicals in a kindergarten... ah!! never mind. Now, I should complete our pilgrimage and take you and some
                                other humans as offering to the god of (*&%$ before we nuke this entire planet. " The alien captain pulled out his weapon.
                                
                                "So your pilgrimage is to kidnap warriors from other planets before destroying them, what an envil civilization, I will 
                                not let you do that!"
                                
                                """, "You returned to the classroom, the alien vessel is gone, there are just broken glasses and swivel chairs swiveling"], self.a4)
        self.map = [self.tim, self.cr1, self.cr2, self.cr3, self.cr4]


    '''
    buy items at tim horton
    display items at tim horton
    ask player for what to order
    update player's health, power and money
    '''
    def buyItem(self):
        if self.p1.locate() == 0:
            for i in self.tim.getInv():
                print("""
                """,i.display())
            order = input("What do you want to order today? ")
            result = self.tim.checkStock(order)
            if result != 1 and result != 2 and result != 3:
                if self.p1.buy() >= result.getprice():
                    self.p1.pay(result.getprice())
                    self.tim.sell(result)
                    print("""
                                  You sat down and enjoyed the {}""".format(result.getname()))
                    print("""                           <(▰˘◡˘▰)>
                                  You feel better now, your health and power are restored""")
                    result.addhealth(self.p1)
                    result.addpower(self.p1)
                else:
                    print("""
                         Oh no you don't have enough money
                    """)

            elif result == 1:
                print("""
                               Sorry, we sold out {}, do you want to 3) order something else?""".format(order))
            elif result == 2:
                print("""
                                  Sorry, can you say again?""")
            elif result == 3:
                self.sparklance.check()
                if self.sparklance.getstatus() == 2:
                    print(self.sparklance.getplot()[1])
                    print(self.sparklance.getplot()[0])
                    self.sparklance.betaTransform(self.p1)
                    self.sparklance.getSpacium(self.p1)
                    self.p1.transform()
                else:
                    print("""
                                  Sorry, we only have breakfast at this branch""")
           
        else:
            print("""
                                  There is no store here""")



    '''
    4 fighting locations in total
    0 alien at location 0
    1 alien on the first floor
    1 alien on the second floor
    and 2 aliens on the third floor
    utilize random method to get random advantages for both player and allien
    calculate the fight result
    output scripture and fighting result depends on player's identity and calculated power
    unlock the level, and get the moeny when alien at that level is dead
    '''                
    def fight(self):
        if self.p1.locate() == 0:
            print("""
                                   There is no one to fight here""")
        else:
            for r in self.map[1:]:
                if r.getLoc() == self.p1.locate():
                    if self.p1.getidentity() == 0:
                        print("""
                                                (ง •̀_•́)ง   VS   (╰_╯)
                                                you are fighting the alien""")
                    else:
                        print("""
                                                <(o|o)>   VS   (╰_╯)
                                                you are fighting the alien""")
                        
                    dice1 = random.randint(1,6)
                    dice2 = random.randint(1,6)
                    result = int(self.p1.fight()*(1+dice1/10) - r.getAlien().fight()*(1+dice2/10))
                    print("""
                                             Your advantage: {}    Alien advantage: {}""".format(dice1, dice2))
                    if result > 0:
                        r.getAlien().wound(result)
                        if self.p1.getidentity() == 0:
                            print("""
                                               (╯°口°)╯︵ ┻━┻  {}
                                               you inflicted {} damage to the alien""".format( r.getAlien().display(),result))
                        else:
                            print("""
                                               ( o|o)ノ三三三三三三三三三  {}
                                               you inflicted {} damage to the alien""".format( r.getAlien().display(),result))

                        if r.getAlien().isdead():
                            print("""
                                             └(^o^)┘ you defeated the alien""")
                            self.p1.getMoney(r.getAlien().dropMoney())
                            r.getAlien().die()
                            r.unlock()
                    elif result == 0:
                        print("""
                                            (ꐦಠ ಠ)     {}
                                            The alien is as strong as you
                                            You had a draw with the alien""".format(r.getAlien().display()))
                    else:
                        print("""
                                            _(:\ 」∠)_      {}
                                            The alien knocked you off, inflicting {} damage on you
                                            you can still stand up to fight""".format(r.getAlien().display(), abs(result)))
                        self.p1.wound(abs(result))
                        if self.p1.isdefeated():
                            self.p1.defeat()


                        

    '''
    if player arrive at 4 location and alien's power at 4 location is 0, then player win the game
    '''
    def win(self):
        if self.p1.locate() == self.p1.endPos and self.cr4.getAlien().health == 0:
            print("""
                        You took your laptop from the dead alien captain, it still works! Your lab files are intact!
                        With its commander dead, the alien vessel fled to the space, they dare not to cross Earth again.
                        Without further delay, you submitted the lab to canvas and headed back home. Just another day in
                        the MDS program. -end
                        You can continue to walk around or just go back home
            """)



    '''
    advance one level up when the level is unlocked, display the alien and set the location status visited
    report error when the level us locked
    '''
    def advance(self):
        flag = 0
        for r in self.map:
            if flag == 1:
                break
            else:
                if r.getLoc() == self.p1.locate():
                    flag = 1
                    if r.islocked():
                        print("""
                              as you approach, the alien shouted at you, you can't advance further""")
                    else:
                        self.p1.advance()
                        for l in self.map: 
                            if l.getLoc() == self.p1.locate():
                                try:
                                    print("""
                                        {} 
                                        (/ﾟДﾟ)/ you found an {} ({}) in front of you
                                        """.format(l.getplot(), l.getAlien().getName(), str(l.getAlien().display())))
                                except Exception as ex:
                                    print(ex)
                                l.visit()
                            
    
            
    '''
    display the scripture and ask the player for 5 options of next steps
    option 1 advance
    option 2 retreat
    option 3 buy item at tim horton
    option 4 fight against alien
    option 0 exit the game
    '''
    def play(self):
        command = 0
        print(self.script0)
        while 1:
            self.win()
            print("____________________________________________________")
            print("1) advance  2) retreat  3)buy item 4) fight 0) exit")
            self.p1.display()
            try:
                command = int(input("what is your next move?"))
                if command > 4:
                    raise Exception

                if command == 1:
                    self.advance()
                elif command == 2:
                    self.p1.retreat()
                    for r in self.map:
                        if r.getLoc() == self.p1.locate():
                            try:
                                print("""
                                {}
                                """.format(r.getplot()))
                            except Exception as ex:
                                print(ex)
                elif command == 0:
                    print("game over")
                    break
                elif command == 3:
                    self.buyItem()

                elif command == 4:
                    self.fight()
                    if self.p1.isdefeated():
                        break
            except(ValueError):
                print("""
                          Please enter a number""")
            except Exception as ex:
                print(ex)
                print("""
                          Please select from available Options""")




import unittest
import sys
sys.path.insert(0, sys.path[0][:-4])
from MDS_Big_Adventure.item.food import item

class TestItem(unittest.TestCase):
    
    @classmethod
    def setUpClass(cls):
        print("")
    
    def setUp(self):
        self.i = item("bacon",20,30,2,2)
        
    def test_gethp(self):
        self.assertEqual(self.i.gethp(),20)
        self.assertNotEqual(self.i.gethp(),"bacon")
        self.assertNotEqual(self.i.gethp(),30)
        self.assertNotEqual(self.i.gethp(),2)
        
    def test_getprice(self):
        self.assertEqual(self.i.getprice(),2)
        self.assertNotEqual(self.i.getprice(),200)
        self.assertNotEqual(self.i.getprice(),-1)
        self.assertNotEqual(self.i.getprice(),100)
        
    def test_getpo(self):
        self.assertEqual(self.i.getpo(),30)
        self.assertNotEqual(self.i.getpo(),"bacon")
        self.assertNotEqual(self.i.getpo(),20)
        self.assertNotEqual(self.i.getpo(),2)
    def tearDown(self):
        print("")
        
    @classmethod
    def tearDownClass(cls):
         print("")
         
    unittest.main(argv=[''], verbosity=2, exit=False)   




class item():

    #initialize
    def __init__(self, name, hp, po, price, amount):
        self.__name = name
        self.__hp = hp
        self.__po = po
        self.__price = price
        self.__amount = amount

    #return health bonus of the item
    def gethp(self):
        return self.__hp

    #return power bonus of the item
    def getpo(self):
        return self.__po

    #return item price
    def getprice(self):
        return self.__price

    #add health to the player
    def addhealth(self, p):
        p.health += self.gethp()

    #add power to the player
    def addpower(self, p):
        p.power += self.getpo()  

    #return the item name
    def getname(self):
        return self.__name
    
    #return item amount
    def getamount(self):
        return self.__amount

    #set item amount
    def setamount(self, value):
        self.__amount = value

    #display item name and price
    def display(self):
        return "{}, price {}".format(self.getname(), self.getprice())




from .character import character
from ..exceptions.InvalidPriceException import InvalidPriceException

#inheritence from character
class player(character):

    #initial location at tim horton

    #4 fighting locations in total
    #one alien on the first floor
    #one alien on the second floor
    #and two aliens on the third floor
    endPos = 4

    #initialize
    def __init__(self, health, power,money, identity):
        character.__init__(self, health, power)
        self.money = money
        self.__identity = identity
        self.position = 0

    #advance one floor if player does not reach to top floor
    def advance(self):
        try:
            if self.position < self.endPos:
                self.position += 1

                print("""
                       վ'ᴗ' ի you advanced""")
            else:
                print("""
                       (;´・`)> Can't advance further""")
        except UnicodeEncodeError as ex:
            print(ex)
            print("can't process the emoticons")

    #retreat one floor if player does not reach to tim horton
    def retreat(self):
        try:
            if self.position >= 1:
                self.position -=1
                print("""
                      ヽ(*。>Д<)o゜ you retreated""")
            else:
                print("""
                       (;´・`)> There is no other place to go""")
        except UnicodeEncodeError as ex:
            print(ex)
            print("can't process the emoticons")

    #return current position 
    def locate(self):
        return self.position

    #if at tim horton return current money
    def buy(self):
        if self.locate() == 0:
            return self.money

    #print the lines when player is defeated
    def defeat(self):
        print(
        """
                    No... you can't get up any more...
                    
                    ~~~~~~~~~~~~Sad BGM playing~~~~~~~~~~~~~~~~~
                     
                    You are defeated by the alien...
                    Before you lose consciousness, you felt the alien lifting you up.
                    Where would he take you? You may never have a chance to find out...
                    
                    What would become of the fate of humanity? 
                    ~~~~~~~~~~~~flashback of good memories~~~~~~~~
                    
                    But more importly, your assignment... you are going to miss it...
                    
                    You lost consciousness...
                    
                    Game over
                    
                    
        """)

        
    #display health, power and money according to player's identity  
    def display(self):
        if self.__identity == 0:
            print("('ᴗ')", character.display(self), " Money: {}".format(self.showMoney()))
        elif self.__identity == 1:
            print(".( o|o)/", character.display(self), " Money: {}".format(self.showMoney()))

    #get the money from the dead alien
    def getMoney(self, money):
        try:
            print("""
                                    You reclaimed ${} from the dead alien, they must have stolen these cash
                                    from someone's backpack""".format(money))
            self.money += money
        except(TypeError):
            print("You found something strange from the alien")
    
    #return current money value
    def showMoney(self):
        return self.money

    #deduct money when buying items in tim horton
    def pay(self,price):
        try:
            if price < 0:
                raise InvalidPriceException
            self.money -= price
        except TypeError:
            return "Price has to be a number"
        except(InvalidPriceException):
            return "Price can't be negative"

    #upgrade player identity when player gets spark lance
    def transform(self):
        self.__identity = 1

    #check if player is dead
    def isdefeated(self):
        if self.health == 0:
            return True

    #return player's identity
    def getidentity(self):
        return self.__identity
